@@ -34,7 +34,7 @@ Module to provide MySQL compatibility to salt.
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import time
 import logging
 import re
@@ -43,10 +43,12 @@ import shlex
 import os
 
 # Import salt libs
-import salt.utils
+import salt.utils.data
+import salt.utils.files
+import salt.utils.stringutils
 
 # Import third party libs
-import salt.ext.six as six
+from salt.ext import six
 # pylint: disable=import-error
 from salt.ext.six.moves import range, zip  # pylint: disable=no-name-in-module,redefined-builtin
 try:
@@ -79,7 +81,11 @@ __grants__ = [
     'ALL PRIVILEGES',
     'ALTER',
     'ALTER ROUTINE',
+    'BACKUP_ADMIN',
+    'BINLOG_ADMIN',
+    'CONNECTION_ADMIN',
     'CREATE',
+    'CREATE ROLE',
     'CREATE ROUTINE',
     'CREATE TABLESPACE',
     'CREATE TEMPORARY TABLES',
@@ -87,26 +93,37 @@ __grants__ = [
     'CREATE VIEW',
     'DELETE',
     'DROP',
+    'DROP ROLE',
+    'ENCRYPTION_KEY_ADMIN',
     'EVENT',
     'EXECUTE',
     'FILE',
     'GRANT OPTION',
+    'GROUP_REPLICATION_ADMIN',
     'INDEX',
     'INSERT',
     'LOCK TABLES',
+    'PERSIST_RO_VARIABLES_ADMIN',
     'PROCESS',
     'REFERENCES',
     'RELOAD',
     'REPLICATION CLIENT',
     'REPLICATION SLAVE',
+    'REPLICATION_SLAVE_ADMIN',
+    'RESOURCE_GROUP_ADMIN',
+    'RESOURCE_GROUP_USER',
+    'ROLE_ADMIN',
     'SELECT',
+    'SET_USER_ID',
     'SHOW DATABASES',
     'SHOW VIEW',
     'SHUTDOWN',
     'SUPER',
+    'SYSTEM_VARIABLES_ADMIN',
     'TRIGGER',
     'UPDATE',
-    'USAGE'
+    'USAGE',
+    'XA_RECOVER_ADMIN'
 ]
 
 __ssl_options_parameterized__ = [
@@ -119,6 +136,52 @@ __ssl_options__ = __ssl_options_parameterized__ + [
     'X509'
 ]
 
+__all_privileges__ = [
+    'ALTER',
+    'ALTER ROUTINE',
+    'BACKUP_ADMIN',
+    'BINLOG_ADMIN',
+    'CONNECTION_ADMIN',
+    'CREATE',
+    'CREATE ROLE',
+    'CREATE ROUTINE',
+    'CREATE TABLESPACE',
+    'CREATE TEMPORARY TABLES',
+    'CREATE USER',
+    'CREATE VIEW',
+    'DELETE',
+    'DROP',
+    'DROP ROLE',
+    'ENCRYPTION_KEY_ADMIN',
+    'EVENT',
+    'EXECUTE',
+    'FILE',
+    'GROUP_REPLICATION_ADMIN',
+    'INDEX',
+    'INSERT',
+    'LOCK TABLES',
+    'PERSIST_RO_VARIABLES_ADMIN',
+    'PROCESS',
+    'REFERENCES',
+    'RELOAD',
+    'REPLICATION CLIENT',
+    'REPLICATION SLAVE',
+    'REPLICATION_SLAVE_ADMIN',
+    'RESOURCE_GROUP_ADMIN',
+    'RESOURCE_GROUP_USER',
+    'ROLE_ADMIN',
+    'SELECT',
+    'SET_USER_ID',
+    'SHOW DATABASES',
+    'SHOW VIEW',
+    'SHUTDOWN',
+    'SUPER',
+    'SYSTEM_VARIABLES_ADMIN',
+    'TRIGGER',
+    'UPDATE',
+    'XA_RECOVER_ADMIN'
+]
+
 r'''
 DEVELOPER NOTE: ABOUT arguments management, escapes, formats, arguments and
 security of SQL.
@@ -330,7 +393,7 @@ def _connect(**kwargs):
     try:
         dbc = MySQLdb.connect(**connargs)
     except MySQLdb.OperationalError as exc:
-        err = 'MySQL Error {0}: {1}'.format(*exc)
+        err = 'MySQL Error {0}: {1}'.format(*exc.args)
         __context__['mysql.error'] = err
         log.error(err)
         return None
@@ -370,7 +433,7 @@ def _grant_to_tokens(grant):
             - grant: [grant1, grant2] (ala SELECT, USAGE, etc)
             - database: MySQL DB
     '''
-    log.debug('_grant_to_tokens entry \'{0}\''.format(grant))
+    log.debug('_grant_to_tokens entry \'%s\'', grant)
     dict_mode = False
     if isinstance(grant, dict):
         dict_mode = True
@@ -414,7 +477,6 @@ def _grant_to_tokens(grant):
     position_tracker = 1  # Skip the initial 'GRANT' word token
     database = ''
     phrase = 'grants'
-    #log.debug('_grant_to_tokens lex analysis \'{0}\''.format(exploded_grant))
 
     for token in exploded_grant[position_tracker:]:
 
@@ -484,12 +546,11 @@ def _grant_to_tokens(grant):
             user = user.strip("'")
             host = host.strip("'")
         log.debug(
-            'grant to token \'{0}\'::\'{1}\'::\'{2}\'::\'{3}\''.format(
+            'grant to token \'%s\'::\'%s\'::\'%s\'::\'%s\'',
                 user,
                 host,
                 grant_tokens,
                 database
-            )
         )
     except UnboundLocalError:
         host = ''
@@ -541,10 +602,10 @@ def _execute(cur, qry, args=None):
     '''
     if args is None or args == {}:
         qry = qry.replace('%%', '%')
-        log.debug('Doing query: {0}'.format(qry))
+        log.debug('Doing query: %s', qry)
         return cur.execute(qry)
     else:
-        log.debug('Doing query: {0} args: {1} '.format(qry, repr(args)))
+        log.debug('Doing query: %s args: %s ', qry, repr(args))
         return cur.execute(qry, args)
 
 
@@ -644,11 +705,11 @@ def query(database, query, **connection_args):
         return {}
     cur = dbc.cursor()
     start = time.time()
-    log.debug('Using db: {0} to run query {1}'.format(database, query))
+    log.debug('Using db: %s to run query %s', database, query)
     try:
         affected = _execute(cur, query)
     except MySQLdb.OperationalError as exc:
-        err = 'MySQL Error {0}: {1}'.format(*exc)
+        err = 'MySQL Error {0}: {1}'.format(*exc.args)
         __context__['mysql.error'] = err
         log.error(err)
         return False
@@ -713,10 +774,10 @@ def file_query(database, file_name, **connection_args):
         file_name = __salt__['cp.cache_file'](file_name)
 
     if os.path.exists(file_name):
-        with salt.utils.fopen(file_name, 'r') as ifile:
-            contents = ifile.read()
+        with salt.utils.files.fopen(file_name, 'r') as ifile:
+            contents = salt.utils.stringutils.to_unicode(ifile.read())
     else:
-        log.error('File "{0}" does not exist'.format(file_name))
+        log.error('File "%s" does not exist', file_name)
         return False
 
     query_string = ""
@@ -745,7 +806,7 @@ def file_query(database, file_name, **connection_args):
                 ret['results'].append(query_result['results'])
             if 'rows affected' in query_result:
                 ret['rows affected'] += query_result['rows affected']
-    ret['query time']['human'] = str(round(float(ret['query time']['raw']), 2)) + 's'
+    ret['query time']['human'] = six.text_type(round(float(ret['query time']['raw']), 2)) + 's'
     ret['query time']['raw'] = round(float(ret['query time']['raw']), 5)
 
     # Remove empty keys in ret
@@ -773,7 +834,7 @@ def status(**connection_args):
     try:
         _execute(cur, qry)
     except MySQLdb.OperationalError as exc:
-        err = 'MySQL Error {0}: {1}'.format(*exc)
+        err = 'MySQL Error {0}: {1}'.format(*exc.args)
         __context__['mysql.error'] = err
         log.error(err)
         return {}
@@ -804,13 +865,13 @@ def version(**connection_args):
     try:
         _execute(cur, qry)
     except MySQLdb.OperationalError as exc:
-        err = 'MySQL Error {0}: {1}'.format(*exc)
+        err = 'MySQL Error {0}: {1}'.format(*exc.args)
         __context__['mysql.error'] = err
         log.error(err)
         return ''
 
     try:
-        return cur.fetchone()[0]
+        return salt.utils.data.decode(cur.fetchone()[0])
     except IndexError:
         return ''
 
@@ -837,7 +898,7 @@ def slave_lag(**connection_args):
     try:
         _execute(cur, qry)
     except MySQLdb.OperationalError as exc:
-        err = 'MySQL Error {0}: {1}'.format(*exc)
+        err = 'MySQL Error {0}: {1}'.format(*exc.args)
         __context__['mysql.error'] = err
         log.error(err)
         return -3
@@ -922,7 +983,7 @@ def db_list(**connection_args):
     try:
         _execute(cur, qry)
     except MySQLdb.OperationalError as exc:
-        err = 'MySQL Error {0}: {1}'.format(*exc)
+        err = 'MySQL Error {0}: {1}'.format(*exc.args)
         __context__['mysql.error'] = err
         log.error(err)
         return []
@@ -952,7 +1013,7 @@ def alter_db(name, character_set=None, collate=None, **connection_args):
         return []
     cur = dbc.cursor()
     existing = db_get(name, **connection_args)
-    qry = 'ALTER DATABASE {0} CHARACTER SET {1} COLLATE {2};'.format(
+    qry = 'ALTER DATABASE `{0}` CHARACTER SET {1} COLLATE {2};'.format(
         name.replace('%', r'\%').replace('_', r'\_'),
         character_set or existing.get('character_set'),
         collate or existing.get('collate'))
@@ -998,7 +1059,7 @@ def db_tables(name, **connection_args):
         salt '*' mysql.db_tables 'database'
     '''
     if not db_exists(name, **connection_args):
-        log.info('Database \'{0}\' does not exist'.format(name))
+        log.info('Database \'%s\' does not exist', name)
         return False
 
     dbc = _connect(**connection_args)
@@ -1011,7 +1072,7 @@ def db_tables(name, **connection_args):
     try:
         _execute(cur, qry)
     except MySQLdb.OperationalError as exc:
-        err = 'MySQL Error {0}: {1}'.format(*exc)
+        err = 'MySQL Error {0}: {1}'.format(*exc.args)
         __context__['mysql.error'] = err
         log.error(err)
         return []
@@ -1046,7 +1107,7 @@ def db_exists(name, **connection_args):
     try:
         _execute(cur, qry, args)
     except MySQLdb.OperationalError as exc:
-        err = 'MySQL Error {0}: {1}'.format(*exc)
+        err = 'MySQL Error {0}: {1}'.format(*exc.args)
         __context__['mysql.error'] = err
         log.error(err)
         return False
@@ -1076,7 +1137,7 @@ def db_create(name, character_set=None, collate=None, **connection_args):
     '''
     # check if db exists
     if db_exists(name, **connection_args):
-        log.info('DB \'{0}\' already exists'.format(name))
+        log.info('DB \'%s\' already exists', name)
         return False
 
     # db doesn't exist, proceed
@@ -1098,10 +1159,10 @@ def db_create(name, character_set=None, collate=None, **connection_args):
 
     try:
         if _execute(cur, qry, args):
-            log.info('DB \'{0}\' created'.format(name))
+            log.info('DB \'%s\' created', name)
             return True
     except MySQLdb.OperationalError as exc:
-        err = 'MySQL Error {0}: {1}'.format(*exc)
+        err = 'MySQL Error {0}: {1}'.format(*exc.args)
         __context__['mysql.error'] = err
         log.error(err)
     return False
@@ -1119,11 +1180,11 @@ def db_remove(name, **connection_args):
     '''
     # check if db exists
     if not db_exists(name, **connection_args):
-        log.info('DB \'{0}\' does not exist'.format(name))
+        log.info('DB \'%s\' does not exist', name)
         return False
 
     if name in ('mysql', 'information_scheme'):
-        log.info('DB \'{0}\' may not be removed'.format(name))
+        log.info('DB \'%s\' may not be removed', name)
         return False
 
     # db does exists, proceed
@@ -1137,16 +1198,16 @@ def db_remove(name, **connection_args):
     try:
         _execute(cur, qry)
     except MySQLdb.OperationalError as exc:
-        err = 'MySQL Error {0}: {1}'.format(*exc)
+        err = 'MySQL Error {0}: {1}'.format(*exc.args)
         __context__['mysql.error'] = err
         log.error(err)
         return False
 
     if not db_exists(name, **connection_args):
-        log.info('Database \'{0}\' has been removed'.format(name))
+        log.info('Database \'%s\' has been removed', name)
         return True
 
-    log.info('Database \'{0}\' has not been removed'.format(name))
+    log.info('Database \'%s\' has not been removed', name)
     return False
 
 
@@ -1169,7 +1230,7 @@ def user_list(**connection_args):
         qry = 'SELECT User,Host FROM mysql.user'
         _execute(cur, qry)
     except MySQLdb.OperationalError as exc:
-        err = 'MySQL Error {0}: {1}'.format(*exc)
+        err = 'MySQL Error {0}: {1}'.format(*exc.args)
         __context__['mysql.error'] = err
         log.error(err)
         return []
@@ -1203,6 +1264,9 @@ def user_exists(user,
         salt '*' mysql.user_exists 'username' passwordless=True
         salt '*' mysql.user_exists 'username' password_column='authentication_string'
     '''
+    run_verify = False
+    server_version = version(**connection_args)
+    compare_version = '10.2.0' if 'MariaDB' in server_version else '8.0.11'
     dbc = _connect(**connection_args)
     # Did we fail to connect with the user we are checking
     # Its password might have previously change with the same command/state
@@ -1227,23 +1291,30 @@ def user_exists(user,
     args['user'] = user
     args['host'] = host
 
-    if salt.utils.is_true(passwordless):
-        if salt.utils.is_true(unix_socket):
+    if salt.utils.data.is_true(passwordless):
+        if salt.utils.data.is_true(unix_socket):
             qry += ' AND plugin=%(unix_socket)s'
             args['unix_socket'] = 'unix_socket'
         else:
             qry += ' AND ' + password_column + ' = \'\''
     elif password:
-        qry += ' AND ' + password_column + ' = PASSWORD(%(password)s)'
-        args['password'] = str(password)
+        if salt.utils.versions.version_cmp(server_version, compare_version) >= 0:
+            run_verify = True
+        else:
+            _password = password
+            qry += ' AND ' + password_column + ' = PASSWORD(%(password)s)'
+            args['password'] = six.text_type(_password)
     elif password_hash:
         qry += ' AND ' + password_column + ' = %(password)s'
         args['password'] = password_hash
 
+    if run_verify:
+        if not verify_login(user, password, **connection_args):
+            return False
     try:
         _execute(cur, qry, args)
     except MySQLdb.OperationalError as exc:
-        err = 'MySQL Error {0}: {1}'.format(*exc)
+        err = 'MySQL Error {0}: {1}'.format(*exc.args)
         __context__['mysql.error'] = err
         log.error(err)
         return False
@@ -1275,7 +1346,7 @@ def user_info(user, host='localhost', **connection_args):
     try:
         _execute(cur, qry, args)
     except MySQLdb.OperationalError as exc:
-        err = 'MySQL Error {0}: {1}'.format(*exc)
+        err = 'MySQL Error {0}: {1}'.format(*exc.args)
         __context__['mysql.error'] = err
         log.error(err)
         return False
@@ -1333,8 +1404,10 @@ def user_create(user,
         salt '*' mysql.user_create 'username' 'hostname' password_hash='hash'
         salt '*' mysql.user_create 'username' 'hostname' allow_passwordless=True
     '''
+    server_version = version(**connection_args)
+    compare_version = '10.2.0' if 'MariaDB' in server_version else '8.0.11'
     if user_exists(user, host, **connection_args):
-        log.info('User \'{0}\'@\'{1}\' already exists'.format(user, host))
+        log.info('User \'%s\'@\'%s\' already exists', user, host)
         return False
 
     dbc = _connect(**connection_args)
@@ -1351,12 +1424,15 @@ def user_create(user,
     args['host'] = host
     if password is not None:
         qry += ' IDENTIFIED BY %(password)s'
-        args['password'] = str(password)
+        args['password'] = six.text_type(password)
     elif password_hash is not None:
-        qry += ' IDENTIFIED BY PASSWORD %(password)s'
+        if salt.utils.versions.version_cmp(server_version, compare_version) >= 0:
+            qry += ' IDENTIFIED BY %(password)s'
+        else:
+            qry += ' IDENTIFIED BY PASSWORD %(password)s'
         args['password'] = password_hash
-    elif salt.utils.is_true(allow_passwordless):
-        if salt.utils.is_true(unix_socket):
+    elif salt.utils.data.is_true(allow_passwordless):
+        if salt.utils.data.is_true(unix_socket):
             if host == 'localhost':
                 qry += ' IDENTIFIED VIA unix_socket'
             else:
@@ -1371,7 +1447,7 @@ def user_create(user,
     try:
         _execute(cur, qry, args)
     except MySQLdb.OperationalError as exc:
-        err = 'MySQL Error {0}: {1}'.format(*exc)
+        err = 'MySQL Error {0}: {1}'.format(*exc.args)
         __context__['mysql.error'] = err
         log.error(err)
         return False
@@ -1383,7 +1459,7 @@ def user_create(user,
         log.info(msg)
         return True
 
-    log.info('User \'{0}\'@\'{1}\' was not created'.format(user, host))
+    log.info('User \'%s\'@\'%s\' was not created', user, host)
     return False
 
 
@@ -1433,14 +1509,19 @@ def user_chpass(user,
         salt '*' mysql.user_chpass frank localhost password_hash='hash'
         salt '*' mysql.user_chpass frank localhost allow_passwordless=True
     '''
+    server_version = version(**connection_args)
+    compare_version = '10.2.0' if 'MariaDB' in server_version else '8.0.11'
     args = {}
     if password is not None:
-        password_sql = 'PASSWORD(%(password)s)'
+        if salt.utils.versions.version_cmp(server_version, compare_version) >= 0:
+            password_sql = '%(password)s'
+        else:
+            password_sql = 'PASSWORD(%(password)s)'
         args['password'] = password
     elif password_hash is not None:
         password_sql = '%(password)s'
         args['password'] = password_hash
-    elif not salt.utils.is_true(allow_passwordless):
+    elif not salt.utils.data.is_true(allow_passwordless):
         log.error('password or password_hash must be specified, unless '
                   'allow_passwordless=True')
         return False
@@ -1455,43 +1536,55 @@ def user_chpass(user,
         password_column = __password_column(**connection_args)
 
     cur = dbc.cursor()
-    qry = ('UPDATE mysql.user SET ' + password_column + '='
-           + password_sql +
-           ' WHERE User=%(user)s AND Host = %(host)s;')
     args['user'] = user
     args['host'] = host
-    if salt.utils.is_true(allow_passwordless) and \
-            salt.utils.is_true(unix_socket):
+    if salt.utils.versions.version_cmp(server_version, compare_version) >= 0:
+        qry = "ALTER USER %(user)s@%(host)s IDENTIFIED BY %(password)s;"
+    else:
+        qry = ('UPDATE mysql.user SET ' + password_column + '=' + password_sql +
+               ' WHERE User=%(user)s AND Host = %(host)s;')
+    if salt.utils.data.is_true(allow_passwordless) and \
+            salt.utils.data.is_true(unix_socket):
         if host == 'localhost':
-            qry = ('UPDATE mysql.user SET ' + password_column + '='
-                   + password_sql + ', plugin=%(unix_socket)s' +
-                   ' WHERE User=%(user)s AND Host = %(host)s;')
-            args['unix_socket'] = 'unix_socket'
+            args['unix_socket'] = 'auth_socket'
+            if salt.utils.versions.version_cmp(server_version, compare_version) >= 0:
+                qry = "ALTER USER %(user)s@%(host)s IDENTIFIED WITH %(unix_socket)s AS %(user)s;"
+            else:
+                qry = ('UPDATE mysql.user SET ' + password_column + '='
+                       + password_sql + ', plugin=%(unix_socket)s' +
+                       ' WHERE User=%(user)s AND Host = %(host)s;')
         else:
             log.error('Auth via unix_socket can be set only for host=localhost')
     try:
         result = _execute(cur, qry, args)
     except MySQLdb.OperationalError as exc:
-        err = 'MySQL Error {0}: {1}'.format(*exc)
+        err = 'MySQL Error {0}: {1}'.format(*exc.args)
         __context__['mysql.error'] = err
         log.error(err)
         return False
 
-    if result:
+    if salt.utils.versions.version_cmp(server_version, compare_version) >= 0:
         _execute(cur, 'FLUSH PRIVILEGES;')
         log.info(
-            'Password for user \'{0}\'@\'{1}\' has been {2}'.format(
+            'Password for user \'%s\'@\'%s\' has been %s',
                 user, host,
                 'changed' if any((password, password_hash)) else 'cleared'
-            )
         )
         return True
+    else:
+        if result:
+            _execute(cur, 'FLUSH PRIVILEGES;')
+            log.info(
+                'Password for user \'%s\'@\'%s\' has been %s',
+                    user, host,
+                    'changed' if any((password, password_hash)) else 'cleared'
+            )
+            return True
 
     log.info(
-        'Password for user \'{0}\'@\'{1}\' was not {2}'.format(
+        'Password for user \'%s\'@\'%s\' was not %s',
             user, host,
             'changed' if any((password, password_hash)) else 'cleared'
-        )
     )
     return False
 
@@ -1520,16 +1613,16 @@ def user_remove(user,
     try:
         _execute(cur, qry, args)
     except MySQLdb.OperationalError as exc:
-        err = 'MySQL Error {0}: {1}'.format(*exc)
+        err = 'MySQL Error {0}: {1}'.format(*exc.args)
         __context__['mysql.error'] = err
         log.error(err)
         return False
 
     if not user_exists(user, host, **connection_args):
-        log.info('User \'{0}\'@\'{1}\' has been removed'.format(user, host))
+        log.info('User \'%s\'@\'%s\' has been removed', user, host)
         return True
 
-    log.info('User \'{0}\'@\'{1}\' has NOT been removed'.format(user, host))
+    log.info('User \'%s\'@\'%s\' has NOT been removed', user, host)
     return False
 
 
@@ -1568,12 +1661,10 @@ def db_check(name,
         # we need to check all tables
         tables = db_tables(name, **connection_args)
         for table in tables:
-            log.info(
-                'Checking table \'{0}\' in db \'{1}\'..'.format(name, table)
-            )
+            log.info('Checking table \'%s\' in db \'%s\'..', name, table)
             ret.append(__check_table(name, table, **connection_args))
     else:
-        log.info('Checking table \'{0}\' in db \'{1}\'..'.format(name, table))
+        log.info('Checking table \'%s\' in db \'%s\'..', name, table)
         ret = __check_table(name, table, **connection_args)
     return ret
 
@@ -1595,12 +1686,10 @@ def db_repair(name,
         # we need to repair all tables
         tables = db_tables(name, **connection_args)
         for table in tables:
-            log.info(
-                'Repairing table \'{0}\' in db \'{1}\'..'.format(name, table)
-            )
+            log.info('Repairing table \'%s\' in db \'%s\'..', name, table)
             ret.append(__repair_table(name, table, **connection_args))
     else:
-        log.info('Repairing table \'{0}\' in db \'{1}\'..'.format(name, table))
+        log.info('Repairing table \'%s\' in db \'%s\'..', name, table)
         ret = __repair_table(name, table, **connection_args)
     return ret
 
@@ -1622,14 +1711,10 @@ def db_optimize(name,
         # we need to optimize all tables
         tables = db_tables(name, **connection_args)
         for table in tables:
-            log.info(
-                'Optimizing table \'{0}\' in db \'{1}\'..'.format(name, table)
-            )
+            log.info('Optimizing table \'%s\' in db \'%s\'..', name, table)
             ret.append(__optimize_table(name, table, **connection_args))
     else:
-        log.info(
-            'Optimizing table \'{0}\' in db \'{1}\'..'.format(name, table)
-        )
+        log.info('Optimizing table \'%s\' in db \'%s\'..', name, table)
         ret = __optimize_table(name, table, **connection_args)
     return ret
 
@@ -1701,11 +1786,11 @@ def __grant_generate(grant,
     table = db_part[2]
 
     if escape:
-        if dbc is not '*':
+        if dbc != '*':
             # _ and % are authorized on GRANT queries and should get escaped
             # on the db name, but only if not requesting a table level grant
-            dbc = quote_identifier(dbc, for_grants=(table is '*'))
-        if table is not '*':
+            dbc = quote_identifier(dbc, for_grants=(table == '*'))
+        if table != '*':
             table = quote_identifier(table)
     # identifiers cannot be used as values, and same thing for grants
     qry = 'GRANT {0} ON {1}.{2} TO %(user)s@%(host)s'.format(grant, dbc, table)
@@ -1714,9 +1799,9 @@ def __grant_generate(grant,
     args['host'] = host
     if isinstance(ssl_option, list) and len(ssl_option):
         qry += __ssl_option_sanitize(ssl_option)
-    if salt.utils.is_true(grant_option):
+    if salt.utils.data.is_true(grant_option):
         qry += ' WITH GRANT OPTION'
-    log.debug('Grant Query generated: {0} args {1}'.format(qry, repr(args)))
+    log.debug('Grant Query generated: %s args %s', qry, repr(args))
     return {'qry': qry, 'args': args}
 
 
@@ -1732,7 +1817,7 @@ def user_grants(user,
         salt '*' mysql.user_grants 'frank' 'localhost'
     '''
     if not user_exists(user, host, **connection_args):
-        log.info('User \'{0}\'@\'{1}\' does not exist'.format(user, host))
+        log.info('User \'%s\'@\'%s\' does not exist', user, host)
         return False
 
     dbc = _connect(**connection_args)
@@ -1746,13 +1831,13 @@ def user_grants(user,
     try:
         _execute(cur, qry, args)
     except MySQLdb.OperationalError as exc:
-        err = 'MySQL Error {0}: {1}'.format(*exc)
+        err = 'MySQL Error {0}: {1}'.format(*exc.args)
         __context__['mysql.error'] = err
         log.error(err)
         return False
 
     ret = []
-    results = cur.fetchall()
+    results = salt.utils.data.decode(cur.fetchall())
     for grant in results:
         tmp = grant[0].split(' IDENTIFIED BY')[0]
         if 'WITH GRANT OPTION' in grant[0] and 'WITH GRANT OPTION' not in tmp:
@@ -1763,12 +1848,12 @@ def user_grants(user,
 
 
 def grant_exists(grant,
-                database,
-                user,
-                host='localhost',
-                grant_option=False,
-                escape=True,
-                **connection_args):
+                 database,
+                 user,
+                 host='localhost',
+                 grant_option=False,
+                 escape=True,
+                 **connection_args):
     '''
     Checks to see if a grant exists in the database
 
@@ -1779,6 +1864,15 @@ def grant_exists(grant,
         salt '*' mysql.grant_exists \
              'SELECT,INSERT,UPDATE,...' 'database.*' 'frank' 'localhost'
     '''
+
+    server_version = version(**connection_args)
+    if 'ALL' in grant:
+        if salt.utils.versions.version_cmp(server_version, '8.0') >= 0 and \
+           'MariaDB' not in server_version:
+            grant = ','.join([i for i in __all_privileges__])
+        else:
+            grant = 'ALL PRIVILEGES'
+
     try:
         target = __grant_generate(
             grant, database, user, host, grant_option, escape
@@ -1794,22 +1888,38 @@ def grant_exists(grant,
                   'this could also indicate a connection error. Check your configuration.')
         return False
 
-    target_tokens = None
+    # Combine grants that match the same database
+    _grants = {}
     for grant in grants:
+        grant_token = _grant_to_tokens(grant)
+        if grant_token['database'] not in _grants:
+            _grants[grant_token['database']] = {'user': grant_token['user'],
+                                                'database': grant_token['database'],
+                                                'host': grant_token['host'],
+                                                'grant': grant_token['grant']}
+        else:
+            _grants[grant_token['database']]['grant'].extend(grant_token['grant'])
+
+    target_tokens = _grant_to_tokens(target)
+    for database, grant_tokens in _grants.items():
         try:
-            if not target_tokens:  # Avoid the overhead of re-calc in loop
-                target_tokens = _grant_to_tokens(target)
-            grant_tokens = _grant_to_tokens(grant)
-            if grant_tokens['user'] == target_tokens['user'] and \
-                    grant_tokens['database'] == target_tokens['database'] and \
-                    grant_tokens['host'] == target_tokens['host'] and \
-                    set(grant_tokens['grant']) >= set(target_tokens['grant']):
+            _grant_tokens = {}
+            _target_tokens = {}
+
+            _grant_matches = [True if i in grant_tokens['grant']
+                              else False for i in target_tokens['grant']]
+
+            for item in ['user', 'database', 'host']:
+                _grant_tokens[item] = grant_tokens[item].replace('"', '').replace('\\', '').replace('`', '')
+                _target_tokens[item] = target_tokens[item].replace('"', '').replace('\\', '').replace('`', '')
+
+            if _grant_tokens['user'] == _target_tokens['user'] and \
+                    _grant_tokens['database'] == _target_tokens['database'] and \
+                    _grant_tokens['host'] == _target_tokens['host'] and \
+                    all(_grant_matches):
                 return True
             else:
-                log.debug('grants mismatch \'{0}\'<>\'{1}\''.format(
-                    grant_tokens,
-                    target_tokens
-                ))
+                log.debug('grants mismatch \'%s\'<>\'%s\'', grant_tokens, target_tokens)
 
         except Exception as exc:  # Fallback to strict parsing
             log.exception(exc)
@@ -1856,7 +1966,7 @@ def grant_add(grant,
     try:
         _execute(cur, qry['qry'], qry['args'])
     except (MySQLdb.OperationalError, MySQLdb.ProgrammingError) as exc:
-        err = 'MySQL Error {0}: {1}'.format(*exc)
+        err = 'MySQL Error {0}: {1}'.format(*exc.args)
         __context__['mysql.error'] = err
         log.error(err)
         return False
@@ -1864,16 +1974,14 @@ def grant_add(grant,
             grant, database, user, host, grant_option, escape,
             **connection_args):
         log.info(
-            'Grant \'{0}\' on \'{1}\' for user \'{2}\' has been added'.format(
-                grant, database, user
-            )
+            'Grant \'%s\' on \'%s\' for user \'%s\' has been added',
+            grant, database, user
         )
         return True
 
     log.info(
-        'Grant \'{0}\' on \'{1}\' for user \'{2}\' has NOT been added'.format(
-            grant, database, user
-        )
+        'Grant \'%s\' on \'%s\' for user \'%s\' has NOT been added',
+        grant, database, user
     )
     return False
 
@@ -1902,22 +2010,22 @@ def grant_revoke(grant,
 
     grant = __grant_normalize(grant)
 
-    if salt.utils.is_true(grant_option):
+    if salt.utils.data.is_true(grant_option):
         grant += ', GRANT OPTION'
 
     db_part = database.rpartition('.')
     dbc = db_part[0]
     table = db_part[2]
-    if dbc is not '*':
+    if dbc != '*':
         # _ and % are authorized on GRANT queries and should get escaped
         # on the db name, but only if not requesting a table level grant
-        s_database = quote_identifier(dbc, for_grants=(table is '*'))
-    if dbc is '*':
+        s_database = quote_identifier(dbc, for_grants=(table == '*'))
+    if dbc == '*':
         # add revoke for *.*
         # before the modification query send to mysql will looks like
         # REVOKE SELECT ON `*`.* FROM %(user)s@%(host)s
         s_database = dbc
-    if table is not '*':
+    if table != '*':
         table = quote_identifier(table)
     # identifiers cannot be used as values, same thing for grants
     qry = 'REVOKE {0} ON {1}.{2} FROM %(user)s@%(host)s;'.format(
@@ -1932,7 +2040,7 @@ def grant_revoke(grant,
     try:
         _execute(cur, qry, args)
     except MySQLdb.OperationalError as exc:
-        err = 'MySQL Error {0}: {1}'.format(*exc)
+        err = 'MySQL Error {0}: {1}'.format(*exc.args)
         __context__['mysql.error'] = err
         log.error(err)
         return False
@@ -1945,15 +2053,13 @@ def grant_revoke(grant,
                         escape,
                         **connection_args):
         log.info(
-            'Grant \'{0}\' on \'{1}\' for user \'{2}\' has been '
-            'revoked'.format(grant, database, user)
-        )
+            'Grant \'%s\' on \'%s\' for user \'%s\' has been '
+            'revoked', grant, database, user)
         return True
 
     log.info(
-        'Grant \'{0}\' on \'{1}\' for user \'{2}\' has NOT been '
-        'revoked'.format(grant, database, user)
-    )
+        'Grant \'%s\' on \'%s\' for user \'%s\' has NOT been '
+        'revoked', grant, database, user)
     return False
 
 
@@ -1963,17 +2069,20 @@ def processlist(**connection_args):
     "SHOW FULL PROCESSLIST".
 
     Returns: a list of dicts, with each dict representing a process:
+
+    .. code-block:: python
+
         {'Command': 'Query',
-                          'Host': 'localhost',
-                          'Id': 39,
-                          'Info': 'SHOW FULL PROCESSLIST',
-                          'Rows_examined': 0,
-                          'Rows_read': 1,
-                          'Rows_sent': 0,
-                          'State': None,
-                          'Time': 0,
-                          'User': 'root',
-                          'db': 'mysql'}
+        'Host': 'localhost',
+        'Id': 39,
+        'Info': 'SHOW FULL PROCESSLIST',
+        'Rows_examined': 0,
+        'Rows_read': 1,
+        'Rows_sent': 0,
+        'State': None,
+        'Time': 0,
+        'User': 'root',
+        'db': 'mysql'}
 
     CLI Example:
 
@@ -2009,24 +2118,24 @@ def __do_query_into_hash(conn, sql_str):
 
     '''
     mod = sys._getframe().f_code.co_name
-    log.debug('{0}<--({1})'.format(mod, sql_str))
+    log.debug('%s<--(%s)', mod, sql_str)
 
     rtn_results = []
 
     try:
         cursor = conn.cursor()
     except MySQLdb.MySQLError:
-        log.error('{0}: Can\'t get cursor for SQL->{1}'.format(mod, sql_str))
+        log.error('%s: Can\'t get cursor for SQL->%s', mod, sql_str)
         cursor.close()
-        log.debug('{0}-->'.format(mod))
+        log.debug('%s-->', mod)
         return rtn_results
 
     try:
         _execute(cursor, sql_str)
     except MySQLdb.MySQLError:
-        log.error('{0}: try to execute : SQL->{1}'.format(mod, sql_str))
+        log.error('%s: try to execute : SQL->%s', mod, sql_str)
         cursor.close()
-        log.debug('{0}-->'.format(mod))
+        log.debug('%s-->', mod)
         return rtn_results
 
     qrs = cursor.fetchall()
@@ -2042,7 +2151,7 @@ def __do_query_into_hash(conn, sql_str):
         rtn_results.append(row)
 
     cursor.close()
-    log.debug('{0}-->'.format(mod))
+    log.debug('%s-->', mod)
     return rtn_results
 
 
@@ -2065,7 +2174,7 @@ def get_master_status(**connection_args):
 
     '''
     mod = sys._getframe().f_code.co_name
-    log.debug('{0}<--'.format(mod))
+    log.debug('%s<--', mod)
     conn = _connect(**connection_args)
     if conn is None:
         return []
@@ -2076,7 +2185,7 @@ def get_master_status(**connection_args):
     if len(rtnv) == 0:
         rtnv.append([])
 
-    log.debug('{0}-->{1}'.format(mod, len(rtnv[0])))
+    log.debug('%s-->%s', mod, len(rtnv[0]))
     return rtnv[0]
 
 
@@ -2135,7 +2244,7 @@ def get_slave_status(**connection_args):
 
     '''
     mod = sys._getframe().f_code.co_name
-    log.debug('{0}<--'.format(mod))
+    log.debug('%s<--', mod)
     conn = _connect(**connection_args)
     if conn is None:
         return []
@@ -2146,7 +2255,7 @@ def get_slave_status(**connection_args):
     if len(rtnv) == 0:
         rtnv.append([])
 
-    log.debug('{0}-->{1}'.format(mod, len(rtnv[0])))
+    log.debug('%s-->%s', mod, len(rtnv[0]))
     return rtnv[0]
 
 
@@ -2165,7 +2274,7 @@ def showvariables(**connection_args):
 
     '''
     mod = sys._getframe().f_code.co_name
-    log.debug('{0}<--'.format(mod))
+    log.debug('%s<--', mod)
     conn = _connect(**connection_args)
     if conn is None:
         return []
@@ -2174,7 +2283,7 @@ def showvariables(**connection_args):
     if len(rtnv) == 0:
         rtnv.append([])
 
-    log.debug('{0}-->{1}'.format(mod, len(rtnv[0])))
+    log.debug('%s-->%s', mod, len(rtnv[0]))
     return rtnv
 
 
@@ -2193,7 +2302,7 @@ def showglobal(**connection_args):
 
     '''
     mod = sys._getframe().f_code.co_name
-    log.debug('{0}<--'.format(mod))
+    log.debug('%s<--', mod)
     conn = _connect(**connection_args)
     if conn is None:
         return []
@@ -2202,5 +2311,30 @@ def showglobal(**connection_args):
     if len(rtnv) == 0:
         rtnv.append([])
 
-    log.debug('{0}-->{1}'.format(mod, len(rtnv[0])))
+    log.debug('%s-->%s', mod, len(rtnv[0]))
     return rtnv
+
+
+def verify_login(user, password=None, **connection_args):
+    '''
+    Attempt to login using the provided credentials.
+    If successful, return true.  Otherwise, return False.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' mysql.verify_login root password
+    '''
+    # Override the connection args for username and password
+    connection_args['connection_user'] = user
+    connection_args['connection_pass'] = password
+
+    dbc = _connect(**connection_args)
+    if dbc is None:
+        # Clear the mysql.error if unable to connect
+        # if the connection fails, we simply return False
+        if 'mysql.error' in __context__:
+            del __context__['mysql.error']
+        return False
+    return True
