@@ -15,7 +15,7 @@ In light of parted not directly supporting partition IDs, some of this module
 has been written to utilize sfdisk instead. For further information, please
 reference the man page for ``sfdisk(8)``.
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 
 # Import python libs
 import os
@@ -23,9 +23,11 @@ import stat
 import string
 import logging
 
-# Import salt libs
-import salt.utils
+# Import Salt libs
+import salt.utils.path
+import salt.utils.platform
 from salt.exceptions import CommandExecutionError
+from salt.ext import six
 
 log = logging.getLogger(__name__)
 
@@ -48,16 +50,16 @@ def __virtual__():
     Only work on POSIX-like systems, which have parted and lsblk installed.
     These are usually provided by the ``parted`` and ``util-linux`` packages.
     '''
-    if salt.utils.is_windows():
+    if salt.utils.platform.is_windows():
         return (False, 'The parted execution module failed to load '
                 'Windows systems are not supported.')
-    if not salt.utils.which('parted'):
+    if not salt.utils.path.which('parted'):
         return (False, 'The parted execution module failed to load '
                 'parted binary is not in the path.')
-    if not salt.utils.which('lsblk'):
+    if not salt.utils.path.which('lsblk'):
         return (False, 'The parted execution module failed to load '
                 'lsblk binary is not in the path.')
-    if not salt.utils.which('partprobe'):
+    if not salt.utils.path.which('partprobe'):
         return (False, 'The parted execution module failed to load '
                 'partprobe binary is not in the path.')
     return __virtualname__
@@ -92,7 +94,7 @@ def _validate_partition_boundary(boundary):
     '''
     try:
         for unit in VALID_UNITS:
-            if boundary.endswith(unit):
+            if six.text_type(boundary).endswith(unit):
                 return
         int(boundary)
     except Exception:
@@ -388,7 +390,7 @@ def mkfs(device, fs_type):
     else:
         mkfs_cmd = 'mkfs.{0}'.format(fs_type)
 
-    if not salt.utils.which(mkfs_cmd):
+    if not salt.utils.path.which(mkfs_cmd):
         return 'Error: {0} is unavailable.'.format(mkfs_cmd)
     cmd = '{0} {1}'.format(mkfs_cmd, device)
     out = __salt__['cmd.run'](cmd).splitlines()
@@ -528,7 +530,7 @@ def name(device, partition, name):
                 'Invalid characters passed to partition.name'
             )
 
-    cmd = 'parted -m -s {0} name {1} {2}'.format(device, partition, name)
+    cmd = '''parted -m -s {0} name {1} "'{2}'"'''.format(device, partition, name)
     out = __salt__['cmd.run'](cmd).splitlines()
     return out
 
