@@ -1,10 +1,9 @@
 # -*- coding: utf-8 -*-
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import os
 import uuid
-import shutil
 import hashlib
 import logging
 import psutil
@@ -15,13 +14,18 @@ import textwrap
 
 # Import Salt Testing libs
 from tests.support.case import ModuleCase
-from tests.support.helpers import get_unused_localhost_port, skip_if_not_root
+from tests.support.helpers import (
+    get_unused_localhost_port,
+    skip_if_not_root,
+    with_tempfile)
 from tests.support.unit import skipIf
 import tests.support.paths as paths
 
 # Import salt libs
-import salt.ext.six as six
-import salt.utils
+import salt.utils.files
+import salt.utils.path
+import salt.utils.platform
+import salt.utils.stringutils
 
 log = logging.getLogger(__name__)
 
@@ -30,27 +34,53 @@ class CPModuleTest(ModuleCase):
     '''
     Validate the cp module
     '''
-    def test_get_file(self):
+    def run_function(self, *args, **kwargs):
+        '''
+        Ensure that results are decoded
+
+        TODO: maybe move this behavior to ModuleCase itself?
+        '''
+        return salt.utils.data.decode(
+            super(CPModuleTest, self).run_function(*args, **kwargs)
+        )
+
+    @with_tempfile()
+    def test_get_file(self, tgt):
         '''
         cp.get_file
         '''
-        tgt = os.path.join(paths.TMP, 'scene33')
         self.run_function(
                 'cp.get_file',
                 [
                     'salt://grail/scene33',
                     tgt,
                 ])
-        with salt.utils.fopen(tgt, 'r') as scene:
-            data = scene.read()
-            self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
-            self.assertNotIn('bacon', data)
+        with salt.utils.files.fopen(tgt, 'r') as scene:
+            data = salt.utils.stringutils.to_unicode(scene.read())
+        self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
+        self.assertNotIn('bacon', data)
 
-    def test_get_file_templated_paths(self):
+    def test_get_file_to_dir(self):
+        '''
+        cp.get_file
+        '''
+        tgt = os.path.join(paths.TMP, '')
+        self.run_function(
+                'cp.get_file',
+                [
+                    'salt://grail/scene33',
+                    tgt,
+                ])
+        with salt.utils.files.fopen(tgt + 'scene33', 'r') as scene:
+            data = salt.utils.stringutils.to_unicode(scene.read())
+        self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
+        self.assertNotIn('bacon', data)
+
+    @with_tempfile()
+    def test_get_file_templated_paths(self, tgt):
         '''
         cp.get_file
         '''
-        tgt = os.path.join(paths.TMP, 'cheese')
         self.run_function(
             'cp.get_file',
             [
@@ -59,22 +89,19 @@ class CPModuleTest(ModuleCase):
             ],
             template='jinja'
         )
-        with salt.utils.fopen(tgt, 'r') as cheese:
-            data = cheese.read()
-            self.assertIn('Gromit', data)
-            self.assertNotIn('bacon', data)
+        with salt.utils.files.fopen(tgt, 'r') as cheese:
+            data = salt.utils.stringutils.to_unicode(cheese.read())
+        self.assertIn('Gromit', data)
+        self.assertNotIn('bacon', data)
 
-    def test_get_file_gzipped(self):
+    @with_tempfile()
+    def test_get_file_gzipped(self, tgt):
         '''
         cp.get_file
         '''
-        tgt = os.path.join(paths.TMP, 'file.big')
         src = os.path.join(paths.FILES, 'file', 'base', 'file.big')
-        with salt.utils.fopen(src, 'r') as fp_:
-            data = fp_.read()
-            if six.PY3:
-                data = salt.utils.to_bytes(data)
-            hash_str = hashlib.md5(data).hexdigest()
+        with salt.utils.files.fopen(src, 'rb') as fp_:
+            hash_str = hashlib.md5(fp_.read()).hexdigest()
 
         self.run_function(
             'cp.get_file',
@@ -84,13 +111,12 @@ class CPModuleTest(ModuleCase):
             ],
             gzip=5
         )
-        with salt.utils.fopen(tgt, 'r') as scene:
+        with salt.utils.files.fopen(tgt, 'rb') as scene:
             data = scene.read()
-            self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
-            self.assertNotIn('bacon', data)
-            if six.PY3:
-                data = salt.utils.to_bytes(data)
-            self.assertEqual(hash_str, hashlib.md5(data).hexdigest())
+        self.assertEqual(hash_str, hashlib.md5(data).hexdigest())
+        data = salt.utils.stringutils.to_unicode(data)
+        self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
+        self.assertNotIn('bacon', data)
 
     def test_get_file_makedirs(self):
         '''
@@ -106,24 +132,24 @@ class CPModuleTest(ModuleCase):
             makedirs=True
         )
         self.addCleanup(shutil.rmtree, os.path.join(paths.TMP, 'make'), ignore_errors=True)
-        with salt.utils.fopen(tgt, 'r') as scene:
-            data = scene.read()
-            self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
-            self.assertNotIn('bacon', data)
+        with salt.utils.files.fopen(tgt, 'r') as scene:
+            data = salt.utils.stringutils.to_unicode(scene.read())
+        self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
+        self.assertNotIn('bacon', data)
 
-    def test_get_template(self):
+    @with_tempfile()
+    def test_get_template(self, tgt):
         '''
         cp.get_template
         '''
-        tgt = os.path.join(paths.TMP, 'scene33')
         self.run_function(
                 'cp.get_template',
                 ['salt://grail/scene33', tgt],
                 spam='bacon')
-        with salt.utils.fopen(tgt, 'r') as scene:
-            data = scene.read()
-            self.assertIn('bacon', data)
-            self.assertNotIn('spam', data)
+        with salt.utils.files.fopen(tgt, 'r') as scene:
+            data = salt.utils.stringutils.to_unicode(scene.read())
+        self.assertIn('bacon', data)
+        self.assertNotIn('spam', data)
 
     def test_get_dir(self):
         '''
@@ -160,21 +186,21 @@ class CPModuleTest(ModuleCase):
 
     # cp.get_url tests
 
-    def test_get_url(self):
+    @with_tempfile()
+    def test_get_url(self, tgt):
         '''
         cp.get_url with salt:// source given
         '''
-        tgt = os.path.join(paths.TMP, 'scene33')
         self.run_function(
             'cp.get_url',
             [
                 'salt://grail/scene33',
                 tgt,
             ])
-        with salt.utils.fopen(tgt, 'r') as scene:
-            data = scene.read()
-            self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
-            self.assertNotIn('bacon', data)
+        with salt.utils.files.fopen(tgt, 'r') as scene:
+            data = salt.utils.stringutils.to_unicode(scene.read())
+        self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
+        self.assertNotIn('bacon', data)
 
     def test_get_url_makedirs(self):
         '''
@@ -190,10 +216,10 @@ class CPModuleTest(ModuleCase):
                 makedirs=True
             )
         self.addCleanup(shutil.rmtree, os.path.join(paths.TMP, 'make'), ignore_errors=True)
-        with salt.utils.fopen(tgt, 'r') as scene:
-            data = scene.read()
-            self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
-            self.assertNotIn('bacon', data)
+        with salt.utils.files.fopen(tgt, 'r') as scene:
+            data = salt.utils.stringutils.to_unicode(scene.read())
+        self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
+        self.assertNotIn('bacon', data)
 
     def test_get_url_dest_empty(self):
         '''
@@ -204,10 +230,10 @@ class CPModuleTest(ModuleCase):
             [
                 'salt://grail/scene33',
             ])
-        with salt.utils.fopen(ret, 'r') as scene:
-            data = scene.read()
-            self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
-            self.assertNotIn('bacon', data)
+        with salt.utils.files.fopen(ret, 'r') as scene:
+            data = salt.utils.stringutils.to_unicode(scene.read())
+        self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
+        self.assertNotIn('bacon', data)
 
     def test_get_url_no_dest(self):
         '''
@@ -235,23 +261,39 @@ class CPModuleTest(ModuleCase):
             ])
         self.assertEqual(ret, False)
 
-    def test_get_url_https(self):
+    def test_get_url_to_dir(self):
+        '''
+        cp.get_url with salt:// source
+        '''
+        tgt = os.path.join(paths.TMP, '')
+        self.run_function(
+                'cp.get_url',
+                [
+                    'salt://grail/scene33',
+                    tgt,
+                ])
+        with salt.utils.files.fopen(tgt + 'scene33', 'r') as scene:
+            data = salt.utils.stringutils.to_unicode(scene.read())
+        self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
+        self.assertNotIn('bacon', data)
+
+    @with_tempfile()
+    def test_get_url_https(self, tgt):
         '''
         cp.get_url with https:// source given
         '''
-        tgt = os.path.join(paths.TMP, 'test_get_url_https')
         self.run_function(
             'cp.get_url',
             [
                 'https://repo.saltstack.com/index.html',
                 tgt,
             ])
-        with salt.utils.fopen(tgt, 'r') as instructions:
-            data = instructions.read()
-            self.assertIn('Bootstrap', data)
-            self.assertIn('Debian', data)
-            self.assertIn('Windows', data)
-            self.assertNotIn('AYBABTU', data)
+        with salt.utils.files.fopen(tgt, 'r') as instructions:
+            data = salt.utils.stringutils.to_unicode(instructions.read())
+        self.assertIn('Bootstrap', data)
+        self.assertIn('Debian', data)
+        self.assertIn('Windows', data)
+        self.assertNotIn('AYBABTU', data)
 
     def test_get_url_https_dest_empty(self):
         '''
@@ -262,12 +304,12 @@ class CPModuleTest(ModuleCase):
             [
                 'https://repo.saltstack.com/index.html',
             ])
-        with salt.utils.fopen(ret, 'r') as instructions:
-            data = instructions.read()
-            self.assertIn('Bootstrap', data)
-            self.assertIn('Debian', data)
-            self.assertIn('Windows', data)
-            self.assertNotIn('AYBABTU', data)
+        with salt.utils.files.fopen(ret, 'r') as instructions:
+            data = salt.utils.stringutils.to_unicode(instructions.read())
+        self.assertIn('Bootstrap', data)
+        self.assertIn('Debian', data)
+        self.assertIn('Windows', data)
+        self.assertNotIn('AYBABTU', data)
 
     def test_get_url_https_no_dest(self):
         '''
@@ -297,10 +339,10 @@ class CPModuleTest(ModuleCase):
                 src,
                 tgt,
             ])
-        with salt.utils.fopen(ret, 'r') as scene:
-            data = scene.read()
-            self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
-            self.assertNotIn('bacon', data)
+        with salt.utils.files.fopen(ret, 'r') as scene:
+            data = salt.utils.stringutils.to_unicode(scene.read())
+        self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
+        self.assertNotIn('bacon', data)
 
     def test_get_url_file_no_dest(self):
         '''
@@ -382,10 +424,10 @@ class CPModuleTest(ModuleCase):
                 [
                     'salt://grail/scene33',
                 ])
-        with salt.utils.fopen(ret, 'r') as scene:
-            data = scene.read()
-            self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
-            self.assertNotIn('bacon', data)
+        with salt.utils.files.fopen(ret, 'r') as scene:
+            data = salt.utils.stringutils.to_unicode(scene.read())
+        self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
+        self.assertNotIn('bacon', data)
 
     def test_cache_files(self):
         '''
@@ -397,17 +439,19 @@ class CPModuleTest(ModuleCase):
                     ['salt://grail/scene33', 'salt://grail/36/scene'],
                 ])
         for path in ret:
-            with salt.utils.fopen(path, 'r') as scene:
-                data = scene.read()
-                self.assertIn('ARTHUR:', data)
-                self.assertNotIn('bacon', data)
+            with salt.utils.files.fopen(path, 'r') as scene:
+                data = salt.utils.stringutils.to_unicode(scene.read())
+            self.assertIn('ARTHUR:', data)
+            self.assertNotIn('bacon', data)
 
-    def test_cache_master(self):
+    @with_tempfile()
+    def test_cache_master(self, tgt):
         '''
         cp.cache_master
         '''
         ret = self.run_function(
                 'cp.cache_master',
+                [tgt],
                 )
         for path in ret:
             self.assertTrue(os.path.exists(path))
@@ -417,15 +461,18 @@ class CPModuleTest(ModuleCase):
         cp.cache_local_file
         '''
         src = os.path.join(paths.TMP, 'random')
-        with salt.utils.fopen(src, 'w+') as fn_:
-            fn_.write('foo')
+        with salt.utils.files.fopen(src, 'w+') as fn_:
+            fn_.write(salt.utils.stringutils.to_str('foo'))
         ret = self.run_function(
                 'cp.cache_local_file',
                 [src])
-        with salt.utils.fopen(ret, 'r') as cp_:
-            self.assertEqual(cp_.read(), 'foo')
+        with salt.utils.files.fopen(ret, 'r') as cp_:
+            self.assertEqual(
+                salt.utils.stringutils.to_unicode(cp_.read()),
+                'foo'
+            )
 
-    @skipIf(not salt.utils.which('nginx'), 'nginx not installed')
+    @skipIf(not salt.utils.path.which('nginx'), 'nginx not installed')
     @skip_if_not_root
     def test_cache_remote_file(self):
         '''
@@ -445,12 +492,12 @@ class CPModuleTest(ModuleCase):
             os.mkdir(dirname)
 
         # Write the temp file
-        with salt.utils.fopen(os.path.join(nginx_root_dir, 'actual_file'), 'w') as fp_:
-            fp_.write(file_contents)
+        with salt.utils.files.fopen(os.path.join(nginx_root_dir, 'actual_file'), 'w') as fp_:
+            fp_.write(salt.utils.stringutils.to_str(file_contents))
 
         # Write the nginx config
-        with salt.utils.fopen(nginx_conf, 'w') as fp_:
-            fp_.write(textwrap.dedent(
+        with salt.utils.files.fopen(nginx_conf, 'w') as fp_:
+            fp_.write(textwrap.dedent(salt.utils.stringutils.to_str(
                 '''\
                 user root;
                 worker_processes 1;
@@ -482,14 +529,14 @@ class CPModuleTest(ModuleCase):
                         }}
                     }}
                 }}'''.format(**locals())
-            ))
+            )))
 
         self.run_function(
             'cmd.run',
             [['nginx', '-c', nginx_conf]],
             python_shell=False
         )
-        with salt.utils.fopen(nginx_pidfile) as fp_:
+        with salt.utils.files.fopen(nginx_pidfile) as fp_:
             nginx_pid = int(fp_.read().strip())
             nginx_proc = psutil.Process(pid=nginx_pid)
             self.addCleanup(nginx_proc.send_signal, signal.SIGQUIT)
@@ -498,8 +545,9 @@ class CPModuleTest(ModuleCase):
             url = url_prefix + (code or 'actual_file')
             log.debug('attempting to cache %s', url)
             ret = self.run_function('cp.cache_file', [url])
-            with salt.utils.fopen(ret) as fp_:
-                cached_contents = fp_.read()
+            self.assertTrue(ret)
+            with salt.utils.files.fopen(ret) as fp_:
+                cached_contents = salt.utils.stringutils.to_unicode(fp_.read())
                 self.assertEqual(cached_contents, file_contents)
 
     def test_list_states(self):
@@ -524,7 +572,7 @@ class CPModuleTest(ModuleCase):
         ret = self.run_function('cp.list_minion')
         found = False
         search = 'grail/scene33'
-        if salt.utils.is_windows():
+        if salt.utils.platform.is_windows():
             search = r'grail\scene33'
         for path in ret:
             if search in path:
@@ -568,35 +616,35 @@ class CPModuleTest(ModuleCase):
                 [
                     'salt://grail/scene33',
                 ])
-        with salt.utils.fopen(path, 'r') as fn_:
+        with salt.utils.files.fopen(path, 'rb') as fn_:
             data = fn_.read()
-            if six.PY3:
-                data = salt.utils.to_bytes(data)
             self.assertEqual(
                 sha256_hash['hsum'], hashlib.sha256(data).hexdigest())
 
-    def test_get_file_from_env_predefined(self):
+    @with_tempfile()
+    def test_get_file_from_env_predefined(self, tgt):
         '''
         cp.get_file
         '''
         tgt = os.path.join(paths.TMP, 'cheese')
         try:
             self.run_function('cp.get_file', ['salt://cheese', tgt])
-            with salt.utils.fopen(tgt, 'r') as cheese:
-                data = cheese.read()
-                self.assertIn('Gromit', data)
-                self.assertNotIn('Comte', data)
+            with salt.utils.files.fopen(tgt, 'r') as cheese:
+                data = salt.utils.stringutils.to_unicode(cheese.read())
+            self.assertIn('Gromit', data)
+            self.assertNotIn('Comte', data)
         finally:
             os.unlink(tgt)
 
-    def test_get_file_from_env_in_url(self):
+    @with_tempfile()
+    def test_get_file_from_env_in_url(self, tgt):
         tgt = os.path.join(paths.TMP, 'cheese')
         try:
             self.run_function('cp.get_file', ['salt://cheese?saltenv=prod', tgt])
-            with salt.utils.fopen(tgt, 'r') as cheese:
-                data = cheese.read()
-                self.assertIn('Gromit', data)
-                self.assertIn('Comte', data)
+            with salt.utils.files.fopen(tgt, 'r') as cheese:
+                data = salt.utils.stringutils.to_unicode(cheese.read())
+            self.assertIn('Gromit', data)
+            self.assertIn('Comte', data)
         finally:
             os.unlink(tgt)
 
@@ -617,3 +665,6 @@ class CPModuleTest(ModuleCase):
             self.assertTrue(os.path.isfile(tgt_cache_file), 'File was not cached on the master')
         finally:
             os.unlink(tgt_cache_file)
+
+    def test_envs(self):
+        self.assertEqual(sorted(self.run_function('cp.envs')), sorted(['base', 'prod']))
