@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 '''
-    :codeauthor: :email:`Rajvi Dhimar <rajvidhimar95@gmail.com>`
+    :codeauthor: Rajvi Dhimar <rajvidhimar95@gmail.com>
 '''
 # Import python libs
-from __future__ import absolute_import, print_function
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import test libs
 from tests.support.mixins import LoaderModuleMockMixin, XMLEqualityMixin
@@ -20,6 +20,8 @@ try:
     from jnpr.junos.utils.config import Config
     from jnpr.junos.utils.sw import SW
     from jnpr.junos.device import Device
+    from jnpr.junos.device import Device
+    import jxmlease  # pylint: disable=unused-import
     HAS_JUNOS = True
 except ImportError:
     HAS_JUNOS = False
@@ -28,7 +30,7 @@ except ImportError:
 import salt.modules.junos as junos
 
 
-@skipIf(not HAS_JUNOS, 'Install junos-eznc to be able to run this test.')
+@skipIf(not HAS_JUNOS, 'The junos-eznc and jxmlease modules are required')
 class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
 
     def setup_loader_modules(self):
@@ -501,7 +503,7 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                     'confirm': 2, '__pub_fun': 'junos.rollback',
                     '__pub_jid': '20170221184518526067', '__pub_tgt': 'mac_min',
                     '__pub_tgt_type': 'glob', '__pub_ret': ''}
-            junos.rollback(2, **args)
+            junos.rollback(id=2, **args)
             mock_rollback.assert_called_with(2)
             mock_commit.assert_called_with(confirm=2)
 
@@ -571,7 +573,7 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
         with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                 patch('jnpr.junos.utils.config.Config.rollback') as mock_rollback, \
-                patch('salt.modules.junos.fopen') as mock_fopen, \
+                patch('salt.utils.files.fopen') as mock_fopen, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff:
             mock_commit_check.return_value = True
             mock_diff.return_value = 'diff'
@@ -592,7 +594,7 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
         with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                 patch('jnpr.junos.utils.config.Config.rollback') as mock_rollback, \
-                patch('salt.modules.junos.fopen') as mock_fopen, \
+                patch('salt.utils.files.fopen') as mock_fopen, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff:
             mock_commit_check.return_value = True
             mock_diff.return_value = None
@@ -646,7 +648,7 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
 
     def test_diff_with_arg(self):
         with patch('jnpr.junos.utils.config.Config.diff') as mock_diff:
-            junos.diff(2)
+            junos.diff(id=2)
             mock_diff.assert_called_with(rb_id=2)
 
     def test_diff_exception(self):
@@ -702,7 +704,7 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
 
     def test_cli_with_format_as_empty_string(self):
         with patch('jnpr.junos.device.Device.cli') as mock_cli:
-            junos.cli('show version', '')
+            junos.cli('show version', format='')
             mock_cli.assert_called_with('show version', 'text', warning=False)
 
     def test_cli(self):
@@ -744,24 +746,6 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
             ret['out'] = False
             self.assertEqual(junos.cli('show version'), ret)
 
-    def test_cli_write_output(self):
-        with patch('salt.modules.junos.fopen') as mock_fopen, \
-                patch('jnpr.junos.device.Device.cli') as mock_cli:
-            mock_cli.return_vale = 'cli text output'
-            args = {'__pub_user': 'root',
-                    '__pub_arg': [{'dest': 'copy/output/here'}],
-                    'dest': 'copy/output/here',
-                    '__pub_fun': 'junos.cli',
-                    '__pub_jid': '20170221182531323467',
-                    '__pub_tgt': 'mac_min',
-                    '__pub_tgt_type': 'glob',
-                    '__pub_ret': ''}
-            ret = dict()
-            ret['message'] = 'cli text output'
-            ret['out'] = True
-            junos.cli('show version', **args)
-            mock_fopen.assert_called_with('copy/output/here', 'w')
-
     def test_shutdown_without_args(self):
         ret = dict()
         ret['message'] = \
@@ -875,8 +859,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -896,8 +880,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -917,8 +901,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -938,8 +922,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -971,8 +955,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -1004,8 +988,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -1033,8 +1017,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
     def test_install_config_load_causes_exception(self):
         with patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -1052,8 +1036,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
     def test_install_config_no_diff(self):
         with patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -1070,10 +1054,10 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
-                patch('salt.modules.junos.fopen') as mock_fopen, \
+                patch('salt.utils.files.fopen') as mock_fopen, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
             mock_getsize.return_value = 10
@@ -1105,10 +1089,10 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
-                patch('salt.modules.junos.fopen') as mock_fopen, \
+                patch('salt.utils.files.fopen') as mock_fopen, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
             mock_getsize.return_value = 10
@@ -1141,8 +1125,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -1174,8 +1158,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
         with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -1193,8 +1177,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
         with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -1213,8 +1197,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -1275,8 +1259,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
 
     def test_install_os(self):
         with patch('jnpr.junos.utils.sw.SW.install') as mock_install, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_getsize.return_value = 10
@@ -1289,8 +1273,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
     def test_install_os_with_reboot_arg(self):
         with patch('jnpr.junos.utils.sw.SW.install') as mock_install, \
                 patch('jnpr.junos.utils.sw.SW.reboot') as mock_reboot, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_getsize.return_value = 10
@@ -1306,8 +1290,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
 
     def test_install_os_pyez_install_throws_exception(self):
         with patch('jnpr.junos.utils.sw.SW.install') as mock_install, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_getsize.return_value = 10
@@ -1321,8 +1305,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
     def test_install_os_with_reboot_raises_exception(self):
         with patch('jnpr.junos.utils.sw.SW.install') as mock_install, \
                 patch('jnpr.junos.utils.sw.SW.reboot') as mock_reboot, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.files.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_getsize.return_value = 10
@@ -1489,29 +1473,26 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
         with patch('jnpr.junos.device.Device.execute') as mock_execute:
             mock_execute.return_value = etree.XML(
                 '<rpc-reply>text rpc reply</rpc-reply>')
-            m = mock_open()
-            with patch('salt.modules.junos.fopen', m, create=True):
-                junos.rpc('get-chassis-inventory', '/path/to/file', 'text')
-                handle = m()
-                handle.write.assert_called_with('text rpc reply')
+            with patch('salt.utils.files.fopen', mock_open(), create=True) as m_open:
+                junos.rpc('get-chassis-inventory', '/path/to/file', format='text')
+                writes = m_open.write_calls()
+                assert writes == ['text rpc reply'], writes
 
     def test_rpc_write_file_format_json(self):
         with patch('jnpr.junos.device.Device.execute') as mock_execute, \
-                patch('salt.modules.junos.json.dumps') as mock_dumps:
+                patch('salt.utils.json.dumps') as mock_dumps:
             mock_dumps.return_value = 'json rpc reply'
-            m = mock_open()
-            with patch('salt.modules.junos.fopen', m, create=True):
+            with patch('salt.utils.files.fopen', mock_open(), create=True) as m_open:
                 junos.rpc('get-chassis-inventory', '/path/to/file', format='json')
-                handle = m()
-                handle.write.assert_called_with('json rpc reply')
+                writes = m_open.write_calls()
+                assert writes == ['json rpc reply'], writes
 
     def test_rpc_write_file(self):
         with patch('salt.modules.junos.jxmlease.parse') as mock_parse, \
                 patch('salt.modules.junos.etree.tostring') as mock_tostring, \
                 patch('jnpr.junos.device.Device.execute') as mock_execute:
             mock_tostring.return_value = 'xml rpc reply'
-            m = mock_open()
-            with patch('salt.modules.junos.fopen', m, create=True):
+            with patch('salt.utils.files.fopen', mock_open(), create=True) as m_open:
                 junos.rpc('get-chassis-inventory', '/path/to/file')
-                handle = m()
-                handle.write.assert_called_with('xml rpc reply')
+                writes = m_open.write_calls()
+                assert writes == ['xml rpc reply'], writes
