@@ -1,7 +1,8 @@
 # -*- coding: utf-8 -*-
 
 # Import python libs
-from __future__ import absolute_import, print_function
+from __future__ import absolute_import, print_function, unicode_literals
+import datetime
 import re
 
 # Import Salt Testing libs
@@ -53,7 +54,7 @@ test_privileges_list_group_csv = (
 @skipIf(NO_MOCK, NO_MOCK_REASON)
 class PostgresTestCase(TestCase, LoaderModuleMockMixin):
     def setup_loader_modules(self):
-        patcher = patch('salt.utils.which', Mock(return_value='/usr/bin/pgsql'))
+        patcher = patch('salt.utils.path.which', Mock(return_value='/usr/bin/pgsql'))
         patcher.start()
         self.addCleanup(patcher.stop)
         return {
@@ -335,6 +336,7 @@ class PostgresTestCase(TestCase, LoaderModuleMockMixin):
                     superuser=False,
                     replication=False,
                     rolepassword='test_role_pass',
+                    valid_until='2042-07-01',
                     groups='test_groups',
                     runas='foo'
                 )
@@ -345,9 +347,10 @@ class PostgresTestCase(TestCase, LoaderModuleMockMixin):
                 call = postgres._run_psql.call_args[0][0][14]
                 self.assertTrue(re.match('CREATE ROLE "testuser"', call))
                 for i in (
-                    'INHERIT NOCREATEDB NOCREATEROLE '
-                    'NOSUPERUSER NOREPLICATION LOGIN UNENCRYPTED PASSWORD'
-                ).split():
+                    'INHERIT', 'NOCREATEDB', 'NOCREATEROLE', 'NOSUPERUSER',
+                    'NOREPLICATION', 'LOGIN', 'UNENCRYPTED', 'PASSWORD',
+                    'VALID UNTIL',
+                ):
                     self.assertTrue(i in call, '{0} not in {1}'.format(i, call))
 
     def test_user_exists(self):
@@ -368,6 +371,8 @@ class PostgresTestCase(TestCase, LoaderModuleMockMixin):
                                    'replication': None,
                                    'password': 'test_password',
                                    'connections': '-1',
+                                   'groups': '',
+                                   'expiry time': '',
                                    'defaults variables': None
                                    }])):
                     ret = postgres.user_exists(
@@ -398,6 +403,8 @@ class PostgresTestCase(TestCase, LoaderModuleMockMixin):
                                    'can login': 't',
                                    'replication': None,
                                    'connections': '-1',
+                                   'groups': '',
+                                   'expiry time': '2017-08-16 08:57:46',
                                    'defaults variables': None
                                }])):
                     ret = postgres.user_list(
@@ -416,7 +423,8 @@ class PostgresTestCase(TestCase, LoaderModuleMockMixin):
                                       'can create roles': True,
                                       'connections': None,
                                       'replication': None,
-                                      'expiry time': None,
+                                      'expiry time': datetime.datetime(
+                                          2017, 8, 16, 8, 57, 46),
                                       'can login': True,
                                       'can update system catalogs': True,
                                       'groups': [],
@@ -464,6 +472,7 @@ class PostgresTestCase(TestCase, LoaderModuleMockMixin):
                     login=True,
                     replication=False,
                     rolepassword='test_role_pass',
+                    valid_until='2017-07-01',
                     groups='test_groups',
                     runas='foo'
                 )
@@ -475,7 +484,8 @@ class PostgresTestCase(TestCase, LoaderModuleMockMixin):
                     re.match(
                         'ALTER ROLE "test_username" WITH  INHERIT NOCREATEDB '
                         'NOCREATEROLE NOREPLICATION LOGIN '
-                        'UNENCRYPTED PASSWORD [\'"]{0,5}test_role_pass[\'"]{0,5};'
+                        'UNENCRYPTED PASSWORD [\'"]{0,5}test_role_pass[\'"]{0,5} '
+                        'VALID UNTIL \'2017-07-01\';'
                         ' GRANT "test_groups" TO "test_username"',
                         postgres._run_psql.call_args[0][0][14]
                     )
